#!/bin/sh

####################### LICENCIA #######################


#This Captive Portal is designed by Jorge Hortelano Otero.
#jorgehortelano@gmail.com
#Copyright (C) 2006 Jorge Hortelano Otero.
#C/Botanico 12, 1. Valencia CP:46008 (Spain).
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.






uso="Type: TocToc.sh <start/stop> [1] without message";

if [ -z "$1" ]
then
	echo $uso;

else


#######################  VARIABLES  #####################

#Ejecutables.
IPT=/sbin/iptables
TC=/sbin/tc

#Direcciones de red completas en linux.
WLAN="192.168.1.0/24"
ETLAN="158.42.55.0/22"

#Nombre de los dispositivos de red en linux.
ETH="eth1"
#WIR="wlan0"
WIR="eth0"

#Parte de la direccion IP que indica la red en la que esta el dispositivo Wifi.
#PWLAN="192.168.0"
PWLAN="192.168.1"

#Puertos aceptados por el servidor.
PUERTOS_ACEPTADOS="9400:9800"
IPS_ACEPTADAS="64.233.183.147 64.233.183.99 64.233.183.104 212.11.63.254 212.11.63.101 212.11.63.100 212.11.63.97 212.11.63.88 212.11.63.73 212.11.63.98 212.11.63.70 212.11.63.82" 

#Carpeta en donde estan todos los documentos php.
DIR_PHP="/Web/TocToc"

#Carpeta en donde se guardan la informacion del cliente.
DIR_CLIENTES="/TocToc_clientes"

#Carpeta donde estn los scripts.
DIR_SCRIPTS="/TocToc_scripts"

#Archivo que contiene los logs de acceso de usuarios al sistema.
LOG="acceso.log"
DIR_LOG="/TocToc_log"

#Tiempo en el que el servidor revisa si existen clientes.
TIEMPO="20"

#Numero de IP del equipo que contiene la pagina TocToc.
TOCTOC="10"

#Rango de los clientes DHCP para los que se quiera aplicar las reglas del servidor.
DHCPmin="195"  #Para el cliente 192.168.1.195
DHCPmax="205" #Hasta el cliente 192.168.1.205

#Tasa superior . Deberia ser el 75% del ancho de banda total de la tarjeta. 
RATEUP=75000
RATEDOWN=100000



if [ $1 == 'start' ]
then

if [ -z $2 ]
then
echo "Starting TocToc System... "
fi
echo "1" > $DIR_SCRIPTS/startstop.txt


#####################  INICIALIZACION  ###################

##FLUSH de reglas.
$IPT -F
$IPT -X
$IPT -Z
$IPT -t nat -F


##Politica por defecto.
$IPT -P INPUT DROP
$IPT -P OUTPUT DROP
$IPT -P FORWARD DROP


##Preparacion de la carpeta de clientes.
rm -f $DIR_CLIENTES/$PWLAN*



#Activacion del NAT y reenvio de paquetes.
$IPT -t nat -A POSTROUTING -s $WLAN -o $ETH -j MASQUERADE
echo "1" > /proc/sys/net/ipv4/ip_forward
echo "1" > /proc/sys/net/ipv4/conf/all/rp_filter



################  PRIVILEGIOS DEL SERVIDOR  ################

#Ethernet
$IPT -A INPUT -i $ETH -j ACCEPT
$IPT -A OUTPUT -o $ETH -j ACCEPT
#$IPT -A OUTPUT -o $WIR -j ACCEPT   #Si se activa esto, se permite que el servidor redireccione tambien el FTP a si mismo en vez de anularlo.

#Loopback
$IPT -A INPUT -i lo -j ACCEPT
$IPT -A OUTPUT -o lo -j ACCEPT

#wlan
$IPT -A INPUT -i $WIR -j ACCEPT
$IPT -A OUTPUT -o $WIR -j ACCEPT



######  SERVICIOS  PROPIOS  OFERTADOS MIENTRAS ACTIVO  ######

sh $DIR_SCRIPTS/Servicios_permitidos.sh $IPT > /dev/null

#Puertos propios
$IPT -A INPUT -p tcp -i $WIR --dport $PUERTOS_ACEPTADOS -j ACCEPT
$IPT -A OUTPUT -p tcp -o $WIR --dport $PUERTOS_ACEPTADOS -j ACCEPT



###############  SERVICIOS  EXTERNOS  GENERALES  ############

# PING
$IPT -A FORWARD -p icmp -j ACCEPT

# DNS
$IPT -A FORWARD  -p udp -s $WLAN --dport 53 -j ACCEPT
$IPT -A FORWARD  -p udp -d $WLAN --sport 53 -j ACCEPT



####################  EQUIPOS ACEPTADOS  ####################

#Se acepta toda la red, asi cuando se quiten las normas de direccionamiento de TocToc, funcionar.

$IPT -A FORWARD  -p tcp -s $WLAN -j ACCEPT
$IPT -A FORWARD  -p tcp -d $WLAN -j ACCEPT



##############  LIBERAMOS EN LA BD LOS CLIENTES ###############

php $DIR_PHP/desconectar_usuarios.php $PWLAN.$i > /dev/null



########### EQUIPOS QUE SE TIENE ACCESO MEDIANTE IP  ############

#Aquellos servidores propios que se pretenden que siempre esten. Como por ejemplo, la ViaMichelin.
for i in $IPS_ACEPTADAS
do
$IPT -t nat -A PREROUTING -p tcp -d $i -j ACCEPT
done

#Aquellos servidores extras que ya se editan con la base de datos.
php $DIR_PHP/activa_servidores.php > /dev/null


############ SI NO ME GUSTA, TE ENVIO A TOCTOC  #############

#Preparo todos los clientes.
for ((i = $DHCPmin; i <= $DHCPmax; i++))
do

	#Lo enviamos a TocToc. Reservamos el puerto 9400 al 9800 para nuestras redirecciones a servicios permitidos. Estos puertos se usarn en la pagina web para indicar que lo queremos. 
	$IPT -t nat -A PREROUTING -p tcp --dport ! $PUERTOS_ACEPTADOS -s $PWLAN.$i  -j DNAT --to-destination $PWLAN.$TOCTOC
	
	
	
	

	#Ponemos su variable
	echo 0 >> $DIR_CLIENTES/$PWLAN.$i.txt;
	chmod 660 $DIR_CLIENTES/$PWLAN.$i.txt;
	chown apache $DIR_CLIENTES/$PWLAN.$i.txt;
	
done



##############  EQUIPOS A LOS QUE SE TIENE ACCESO MEDIANTE PUERTOS ##############

#Ejemplo de como se deberia activar la Gua Michelin mediante puertos.
#$IPT -t nat -A PREROUTING -p TCP --dport 9400 -j DNAT --to-destination 212.11.63.254:80

#Activamos el documento php que activa los servicios que estan en la base de datos.
php $DIR_PHP/activa_servidores_puerto.php > /dev/null
	


##############  ANCHO DE BANDA PARA CADA CLIENTE  ############


#Borramos cualquier cosa que haya de antes.
$TC qdisc del dev $ETH root 2> /dev/null > /dev/null
$TC qdisc del dev $ETH ingress 2> /dev/null > /dev/null
$TC qdisc del dev $ETH root 2> /dev/null > /dev/null


$TC qdisc del dev $WIR root 2> /dev/null > /dev/null
$TC qdisc del dev $WIR ingress 2> /dev/null > /dev/null
$TC qdisc del dev $WIR root 2> /dev/null > /dev/null



#Por que conexion pasa el trafico por defecto. La cola 2 es la cola de los paquetes que salen a internet no clasificados (ACKs).
$TC qdisc add dev $ETH root handle 10: htb default 2
$TC qdisc add dev $WIR root handle 11: htb default 3


#Generacion de la jerarquia. La 10:1 es la que agrupa a todos y la 10:2 la cola por defecto. Idem para 11:1 y 11:20
$TC class add dev $ETH parent 10: classid 10:1 htb rate ${RATEUP}kbit ceil ${RATEUP}kbit
$TC class add dev $WIR parent 11: classid 11:1 htb rate ${RATEDOWN}kbit ceil ${RATEDOWN}kbit

$TC class add dev $ETH parent 10:1 classid 10:2 htb rate 16kbit ceil $[80*$RATEDOWN/100]kbit 
$TC qdisc add dev $ETH parent 10:2 handle 2: sfq perturb 10
$TC filter add dev $ETH parent 10: protocol ip handle 2 fw classid 10:2

$TC class add dev $WIR parent 11:1 classid 11:3 htb rate 8kbit ceil $[80*$RATEUP/100]kbit
$TC qdisc add dev $WIR parent 11:3 handle 22: sfq perturb 10
$TC filter add dev $WIR parent 11: protocol ip handle 4 fw classid 11:3




######  COMPROBAMOS QUE LOS CLIENTES SIGUEN CONECTADOS  ######

#Preparamos un archivo de logs
fecha=$(date +%d" "%m" "%Y" "%T);
chown apache $DIR_CLIENTES
chown apache $DIR_LOG
chmod 770 $DIR_CLIENTES
chmod 770 $DIR_LOG
touch $DIR_LOG/$LOG
chown apache $DIR_LOG/$LOG
chmod 664 $DIR_LOG/$LOG



#Activamos el script para controlar que no se queden los clientes conectados cuando cae la red.
if [[ $(pgrep Controla_client) > 0 ]]
then
	echo "The client control is already activated."
else
	(nohup $DIR_SCRIPTS/Controla_clientes.sh $PWLAN $TOCTOC $DHCPmin $DHCPmax $LOG $TIEMPO $DIR_PHP $DIR_CLIENTES $PUERTOS_ACEPTADOS &)  > /dev/null
fi












#############################################################
#######################        STOP         ############################
#############################################################


else #Del start/stop


if [ $1 == 'stop' ]
then
if [ -z $2 ]
then
echo "Halting TocToc System...";
fi
echo "0" > $DIR_SCRIPTS/startstop.txt


##FLUSH de reglas.
$IPT -F
$IPT -X
$IPT -Z
$IPT -t nat -F


##Politica por defecto.
$IPT -P INPUT DROP
$IPT -P OUTPUT DROP
$IPT -P FORWARD ACCEPT


#Activacion del NAT y reenvio de paquetes.
$IPT -t nat -A POSTROUTING -s $WLAN -o $ETH -j MASQUERADE
echo "1" > /proc/sys/net/ipv4/ip_forward
echo "1" > /proc/sys/net/ipv4/conf/all/rp_filter


################  PRIVILEGIOS DEL SERVIDOR  ################

#Ethernet
$IPT -A INPUT -i $ETH -j ACCEPT
$IPT -A OUTPUT -o $ETH -j ACCEPT

#Loopback
$IPT -A INPUT -i lo -j ACCEPT
$IPT -A OUTPUT -o lo -j ACCEPT


#####  SERVICIOS  PROPIOS  OFERTADOS   MIENTRAS INACTIVO #####

sh $DIR_SCRIPTS/Servicios_permitidos.sh $IPT > /dev/null



#Eliminamos lo que no hace ya falta.

rm -f nohup.out

#Matamos el proceso Controla_clientes.sh
pid=$(pgrep Controla_client)

if [[ $pid > " " ]]
then
	kill -9 $pid
fi


else

echo $uso;
	
	
fi #Del stop

fi #Del start

fi #Del [ -z $1]